/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*
 *	mouse.c - Use the mouse's position in the window to
 *		determine the location of the eye position.
 *
 *	Escape Key			- exit the program
 *	`f' Key				- toggle filled / wireframe cylinder
 *
 *	Left Mousebutton, down		- start moving the eye
 *	Left Mousebutton, up		- stop moving the eye
 *	Mouse pointer location		- position eye
 *	Middle Mousebutton, down	- zoom in
 *	Right Mousebutton, down		- zoom out
 */

#include <GL/gl.h>
#include <GL/glu.h>
#include <aux.h>        /* includes <devices.h> */

#include <stdio.h>

/*  Function Prototypes  */

GLvoid  initialize( GLvoid );
GLvoid  drawScene( GLvoid );
GLvoid  resize( GLsizei, GLsizei );
GLvoid  drawGrid( GLvoid );
GLvoid  filled( GLvoid );
GLvoid  startMove( AUX_EVENTREC * );
GLvoid  stopMove( AUX_EVENTREC * );
GLvoid  mouseMove( AUX_EVENTREC * );
GLvoid  zoomIn( AUX_EVENTREC * );
GLvoid  zoomOut( AUX_EVENTREC * );

void polarView( GLdouble, GLdouble, GLdouble, GLdouble );
void printHelp( char * );

/* Global Variables */

GLfloat     latitude, longitude, radius;
GLint       xStart, yStart;
GLboolean   filledFlag = GL_FALSE;

void
main(int argc, char *argv[])
{
	initialize();

	printHelp(argv[0]);

	auxMainLoop( drawScene );
}

void
printHelp( char *progname )
{
        fprintf(stdout, "\n%s\n\n\\
	Escape key			- exit the program\n\
	'f' key				- filled/wireframe cylinder\n\
	Left Mousebutton, down		- start moving the eye\n\
	Left Mousebutton, up		- stop moving the eye\n\
	Mouse pointer location		- position eye\n\
	Middle Mousebutton, down	- zoom in\n\
	Right Mousebutton, down		- zoom out\n\n", progname);
}

GLvoid
initialize( GLvoid )
{
	GLfloat    maxObjectSize, aspect;
	GLsizei    width, height;

	auxGetScreenSize( &width, &height );
	auxInitPosition( width / 4, height / 4, width / 2, height / 2 );
	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH | AUX_DOUBLE );
	auxInitWindow( "mouse" );
	auxReshapeFunc( resize );

	auxKeyFunc( AUX_ESCAPE, auxQuit );
	auxKeyFunc( AUX_f, filled );

	/* Set up functions using the mouse */
	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEDOWN, startMove );
	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEUP, stopMove );
	auxMouseFunc( AUX_MIDDLEBUTTON, AUX_MOUSEDOWN, zoomIn );
	auxMouseFunc( AUX_RIGHTBUTTON, AUX_MOUSEDOWN, zoomOut );

	glClearColor( 0.0, 0.0, 0.0, 1.0 );
	glClearDepth( 1.0 );

	glEnable( GL_DEPTH_TEST );

	glMatrixMode( GL_PROJECTION );
	aspect = (GLfloat) width / height;
	gluPerspective( 45.0, aspect, 3.0, 7.0 );
	glMatrixMode( GL_MODELVIEW );

	radius = 4.5;
	latitude = 60.0;
	longitude = 0.0;
}

/* Define user input functions for mouse events */
GLvoid
startMove( AUX_EVENTREC *event )
{
	/* Set up function to process mouse movements */
	auxMouseFunc( AUX_MOUSELOC, NULL, mouseMove );

	/* Record where the mouse pointer was when the user pressed the
	 *    left mouse buton */
	xStart = event->data[AUX_MOUSEX];
	yStart = event->data[AUX_MOUSEY];
}

GLvoid
stopMove( AUX_EVENTREC *event )
{
	/* Remove function to process mouse movements */
	auxDeleteMouseFunc( AUX_MOUSELOC, NULL, mouseMove );
}

GLvoid
mouseMove( AUX_EVENTREC *event )
{
	GLint     xPos, yPos;
	GLint     xOffset, yOffset;

	xPos = event->data[AUX_MOUSEX];
	yPos = event->data[AUX_MOUSEY];

	/* Using the stored and current mouse positions, compute how far
	 *    the mouse moved */
	xOffset = xPos - xStart;
	yOffset = yPos - yStart;

	/* Adjust the eye position based on the mouse position */
	longitude += xOffset;
	latitude -= yOffset;

	/* Update the stored mouse position for later use */
	xStart = xPos;
	yStart = yPos;
}

GLvoid
zoomIn( AUX_EVENTREC *event )
{
	radius -= 0.10;
}

GLvoid
zoomOut( AUX_EVENTREC *event )
{
	radius += 0.10;
}

GLvoid
filled( GLvoid )
{
	filledFlag = !filledFlag;
}

GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat     aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLfloat) width / height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 3.0, 7.0 );
	glMatrixMode( GL_MODELVIEW );
}

void
polarView( GLdouble radius, GLdouble twist,  GLdouble latitude,
		GLdouble longitude )
{
	glTranslated( 0.0, 0.0, -radius);
	glRotated( -twist, 0.0, 0.0, 1.0 );
	glRotated( -latitude, 1.0, 0.0, 0.0 );
	glRotated( longitude, 0.0, 0.0, 1.0 );
}

GLvoid
drawScene( GLvoid )
{
	static GLfloat    whiteColor[] = { 1.0, 1.0, 1.0 };
	static GLfloat    redColor[] = { 1.0, 0.0, 0.0 };
	static GLfloat    darkGreenColor[] = { 0.0, 0.5, 0.0 };

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glPushMatrix();

		polarView( radius, 0, latitude, longitude );

		glColor3fv( redColor );
		auxSolidCone( 0.3, 0.6 );

		glColor3fv( darkGreenColor );
		glPushMatrix();
			glTranslatef( 0.8, -0.65, 0.0 );
			glRotatef( 30.0, 1.0, 0.5, 1.0 );
			if ( filledFlag == GL_TRUE )
				auxSolidCylinder( 0.3, 0.6 );
			else
				auxWireCylinder( 0.3, 0.6 );
		glPopMatrix();

		glColor3fv( whiteColor );
		drawGrid();

	glPopMatrix();

	auxSwapBuffers();
}












GLvoid
drawGrid( GLvoid )
{
	GLfloat     x, y;

	glBegin( GL_LINES );
	for ( x = -1.0; x < 1.1; x += 0.1 ) {
		glVertex2f( x, -1.0 );
		glVertex2f( x, 1.0 );
	}
	for ( y = -1.0; y < 1.1; y += 0.1 ) {
		glVertex2f( -1.0, y );
		glVertex2f( 1.0, y );
	}
	glEnd();
}
