/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*
 *  move.c - Set up various functions off of keyboard keys.
 *
 *		Escape Key		- exit the program
 *		`f' Key			- toggle filled / wireframe cylinder
 *		`i' Key			- zoom in
 *		`o' Key			- zoom out
 *		Left Arrow Key		- rotate scene left
 *		Right Arrow Key		- rotate scene right
 *		Down Arrow Key		- rotate scene downwards
 *		Up Arrow Key		- rotate scene upwards
 */

#include <GL/gl.h>
#include <GL/glu.h>
#include <aux.h>        /* includes <auxDevice.h> */

#include <stdio.h>

/*  Function Prototypes  */

GLvoid  initialize( GLvoid );
GLvoid  drawScene( GLvoid );
GLvoid  resize( GLsizei, GLsizei );
GLvoid  drawGrid( GLvoid );
GLvoid  up( GLvoid );
GLvoid  down( GLvoid );
GLvoid  left( GLvoid );
GLvoid  right( GLvoid );
GLvoid  zoomIn( GLvoid );
GLvoid  zoomOut( GLvoid );
GLvoid  filled( GLvoid );

void polarView( GLdouble, GLdouble, GLdouble, GLdouble );
void printHelp( char * );

/* Global Variables */

GLfloat     latitude, longitude, radius;
GLboolean   filledFlag = GL_FALSE;

void
main(int argc, char *argv[])
{
	initialize();

	printHelp(argv[0]);

	auxMainLoop( drawScene );
}

void
printHelp( char *progname )
{
        fprintf(stdout, "\n%s\n\n\\
	Escape key		- exit the program\n\
	`f' Key			- toggle filled / wireframe cylinder\n\
	`i' Key			- zoom in\n\
	`o' Key			- zoom out\n\
	Left Arrow Key		- rotate scene left\n\
	Right Arrow Key		- rotate scene right\n\
	Down Arrow Key		- rotate scene downwards\n\
	Up Arrow Key		- rotate scene upwards\n\n", progname);
}

GLvoid
initialize( GLvoid )
{
	GLfloat    maxObjectSize, aspect;
	GLsizei    width, height;

	auxGetScreenSize( &width, &height );
	auxInitPosition( width / 4, height / 4, width / 2, height / 2 );
	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH | AUX_DOUBLE );

	auxInitWindow( "move" );

	auxReshapeFunc( resize );

	auxKeyFunc( AUX_ESCAPE, auxQuit );

	/* Register user input functions that produce motion */
	auxKeyFunc( AUX_LEFT, left );
	auxKeyFunc( AUX_RIGHT, right );
	auxKeyFunc( AUX_UP, up );
	auxKeyFunc( AUX_DOWN, down );
	auxKeyFunc( AUX_i, zoomIn );
	auxKeyFunc( AUX_o, zoomOut );

	auxKeyFunc( AUX_f, filled );

	glClearColor( 0.0, 0.0, 0.0, 1.0 );
	glClearDepth( 1.0 );

	glEnable( GL_DEPTH_TEST );

	glMatrixMode( GL_PROJECTION );
	aspect = (GLfloat) width / height;
	gluPerspective( 45.0, aspect, 3.0, 7.0 );
	glMatrixMode( GL_MODELVIEW );

	radius = 4.5;
	latitude = 60.0;
	longitude = 0.0;
}

/* Define user input functions */
GLvoid
right( GLvoid )
{
	longitude += 10;
}

GLvoid
left( GLvoid )
{
	longitude -= 10;
}

GLvoid
up( GLvoid )
{
	latitude += 10;
}

GLvoid
down( GLvoid )
{
	latitude -= 10;
}

GLvoid
zoomIn( GLvoid )
{
	radius -= 0.1;
}

GLvoid
zoomOut( GLvoid )
{
	radius += 0.1;
}

GLvoid
filled( GLvoid )
{
	filledFlag = !filledFlag;
}



GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat   aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLfloat) width / height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 3.0, 7.0 );
	glMatrixMode( GL_MODELVIEW );
}

void
polarView( GLdouble radius, GLdouble twist,  GLdouble latitude,
				GLdouble longitude )
{
	glTranslated( 0.0, 0.0, -radius);
	glRotated( -twist, 0.0, 0.0, 1.0 );
	glRotated( -latitude, 1.0, 0.0, 0.0 );
	glRotated( longitude, 0.0, 0.0, 1.0 );
}


GLvoid
drawGrid( GLvoid )
{
	GLfloat     x, y;

	glBegin( GL_LINES );
	for ( x = -1.0; x < 1.1; x += 0.1 ) {
		glVertex2f( x, -1.0 );
		glVertex2f( x, 1.0 );
	}
	for ( y = -1.0; y < 1.1; y += 0.1 ) {
		glVertex2f( -1.0, y );
		glVertex2f( 1.0, y );
	}
	glEnd();
}



GLvoid
drawScene( GLvoid )
{
	static GLfloat    whiteColor[] = { 1.0, 1.0, 1.0 };
	static GLfloat    redColor[] = { 1.0, 0.0, 0.0 };
	static GLfloat    darkGreenColor[] = { 0.0, 0.5, 0.0 };

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glPushMatrix();

		polarView( radius, 0, latitude, longitude );

		glColor3fv( redColor );
		auxSolidCone( 0.3, 0.6 );

		glColor3fv( darkGreenColor );
		glPushMatrix();
			glTranslatef( 0.8, -0.65, 0.0 );
			glRotatef( 30.0, 1.0, 0.5, 1.0 );
			if ( filledFlag == GL_TRUE )
				auxSolidCylinder( 0.3, 0.6 );
			else
				auxWireCylinder( 0.3, 0.6 );
		glPopMatrix();

		glColor3fv( whiteColor );
		drawGrid();

	glPopMatrix();

	auxSwapBuffers();
}
