/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*
 *   vertexNormal.c - Show the affects of lighting based on
 *      vertex normals.  The white lines represents the
 *      direction of each of the vertex normals.
 *
 *            Escape key               - exit the program
 *            Left Mousebutton, down   - start moving the eye
 *            Left Mousebutton, up     - stop moving the eye
 *            Mouse pointer location   - position eye
 */

#include <GL/gl.h>
#include <GL/glu.h>
#include <aux.h>
#include <stdio.h>

/*  Function Prototypes  */

GLvoid  initialize( GLvoid );
GLvoid  drawScene( GLvoid );
GLvoid  drawNormal( GLfloat, GLfloat, GLfloat, GLfloat,
	GLfloat, GLfloat );
GLvoid  resize( GLsizei, GLsizei );
GLvoid  startMove( AUX_EVENTREC * );
GLvoid  stopMove( AUX_EVENTREC * );
GLvoid  mouseMove( AUX_EVENTREC * );

void polarView( GLdouble, GLdouble, GLdouble, GLdouble );
void printHelp( char * );

/* Global Variables */

GLfloat     latitude, longitude, radius;
GLint       xStart, yStart;


void
main(int argc, char *argv[])
{
	initialize();

	printHelp(argv[0]);

	auxMainLoop( drawScene );
}

void
printHelp( char *progname )
{
        fprintf(stdout, "\n%s\n\n\\
	Escape key			- exit the program\n\
	Left Mousebutton, down		- start moving the eye\n\
	Left Mousebutton, up		- stop moving the eye\n\
	Mouse pointer location		- position eye\n\n", progname);
}

GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat     aspect, left, right, bottom, top;

	glViewport( 0, 0, width, height );

	aspect = (GLfloat) width / height;
	if ( aspect < 1.0 ) {
		left = -2.0;
		right = 2.0;
		bottom = -2.0 / aspect;
		top = 2.0 / aspect;
	} else {
		left = -2.0 * aspect;
		right = 2.0 * aspect;
		bottom = -2.0;
		top = 2.0;
	}

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glOrtho( left, right, bottom, top, 3.0, 7.0 );
	glMatrixMode( GL_MODELVIEW );
}

GLvoid
initialize( GLvoid )
{
	GLfloat    aspect;
	GLsizei    width, height;

	auxGetScreenSize( &width, &height );
	auxInitPosition( width / 4, height / 4, width / 2,
		height / 2 );
	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH | AUX_DOUBLE );

	auxInitWindow( "vertexNormal" );

	auxReshapeFunc( resize );

	auxKeyFunc( AUX_ESCAPE, auxQuit );

	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEDOWN, startMove );
	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEUP, stopMove );

	glClearColor( 0.0, 0.0, 0.0, 1.0 );

	glEnable( GL_DEPTH_TEST );

	/* Turn on a default light */
	glEnable( GL_LIGHT0 );

	glMatrixMode( GL_PROJECTION );
	aspect = (GLfloat) width / height;
	glOrtho( -2.0 * aspect, 2.0 * aspect, -2.0, 2.0, 3.0, 7.0 );
	glMatrixMode( GL_MODELVIEW );

	radius = 4.5;
	latitude = 60.0;
	longitude = 0.0;
}


GLvoid
startMove( AUX_EVENTREC *event )
{
	auxMouseFunc( AUX_MOUSELOC, NULL, mouseMove );

	xStart = event->data[AUX_MOUSEX];
	yStart = event->data[AUX_MOUSEY];
}


GLvoid
stopMove( AUX_EVENTREC *event )
{
	auxDeleteMouseFunc( AUX_MOUSELOC, NULL, mouseMove );
}


GLvoid
mouseMove( AUX_EVENTREC *event )
{
	GLint     xPos, yPos;
	GLint     xOffset, yOffset;

	xPos = event->data[AUX_MOUSEX];
	yPos = event->data[AUX_MOUSEY];

	/* Using the stored and current mouse positions, compute how far
	 *    the mouse moved */
	xOffset = xPos - xStart;
	yOffset = yPos - yStart;

	/* Adjust the eye position based on the mouse position */
	longitude += xOffset;
	latitude -= yOffset;

	/* Update the stored mouse position for later use */
	xStart = xPos;
	yStart = yPos;
}

void
polarView( GLdouble radius, GLdouble twist,  GLdouble latitude,
		GLdouble longitude )
{
	glTranslated( 0.0, 0.0, -radius);
	glRotated( -twist, 0.0, 0.0, 1.0 );
	glRotated( -latitude, 1.0, 0.0, 0.0 );
	glRotated( longitude, 0.0, 0.0, 1.0 );
}

GLvoid
drawNormal( GLfloat px, GLfloat py, GLfloat pz,
	GLfloat nx, GLfloat ny, GLfloat nz )
{
	glColor3f( 1.0, 1.0, 1.0 );

	glPushMatrix();
	  glTranslatef( px, py, pz );
	  glBegin( GL_LINES );
	    glVertex3f( 0.0, 0.0, 0.0 );
	    glVertex3f( nx, ny, nz );
	  glEnd();
	glPopMatrix();
}


GLvoid
drawScene( GLvoid )
{
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glPushMatrix();

	  polarView( radius, 0, latitude, longitude );

	  /* Turn on lighting computations */
	  glEnable( GL_LIGHTING );

	  /* Draw a lighted triangle */
	  glBegin( GL_POLYGON );
	    glNormal3f( 0.0, 0.0, 1.0 );
	    glVertex2f( 0.0, 1.0 );
	    glNormal3f( 0.162, 0.566, 0.808 );
	    glVertex2f( 0.866, -0.50 );
	    glNormal3f( 0.455, 0.796, 0.398 );
	    glVertex2f( -0.866, -0.50 );
	  glEnd();

	  /* Turn off lighting computations */
	  glDisable( GL_LIGHTING );

	  /* Draw ( unlighted ) lines representing the vertex normals */
	  drawNormal( 0.0, 1.0, 0.0, 0.0, 0.0, 1.0 );
	  drawNormal( 0.866, -0.50, 0.0, 0.162, 0.566, 0.808 );
	  drawNormal( -0.866, -0.50, 0.0, 0.455, 0.796, 0.398 );
	
	glPopMatrix();

	auxSwapBuffers();
}
