/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*
 *	materials.c - Use material properties to change the color and
 *		reflection models for some objects
 *
 *		Escape key			- exit the program
 *		Left Mousebutton, up		- stop moving the eye
 *		Left Mousebutton, down		- start moving the eye
 *		Mouse pointer location		- position eye
 */

#include <GL/gl.h>
#include <GL/glu.h>
#include <aux.h>
#include <stdio.h>

/*  Function Prototypes  */
GLvoid  initialize( GLvoid );
GLvoid  drawScene( GLvoid );
GLvoid  resize( GLsizei, GLsizei );
GLvoid  startMove( AUX_EVENTREC * );
GLvoid  stopMove( AUX_EVENTREC * );
GLvoid  mouseMove( AUX_EVENTREC * );
GLvoid  drawAxes( GLvoid );

void polarView( GLdouble, GLdouble, GLdouble, GLdouble );
void printHelp( char * );

/* Global Variables */
GLfloat			latitude, longitude, radius;
GLint			xStart, yStart;

GLfloat			fovy = 60.0;	/* Field of view in Y angle */
GLfloat			near = 3.0;	/* Near clipping plane location */
GLfloat			far  = 12.0;	/* Far clipping plane location */

void
main(int argc, char *argv[])
{
	initialize();
	printHelp(argv[0]);
	auxMainLoop( drawScene );
}

GLvoid
initialize( GLvoid )
{
	GLfloat    aspect;
	GLsizei    width, height;

	auxGetScreenSize( &width, &height );
	auxInitPosition( width / 4, height / 4, width / 2,
		height / 2 );
	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH | AUX_DOUBLE );
	auxInitWindow( "materials" );

	auxReshapeFunc( resize );

	auxKeyFunc( AUX_ESCAPE, auxQuit );

	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEDOWN, startMove );
	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEUP, stopMove );

	glClearColor( 0.0, 0.0, 0.0, 1.0 );

	glEnable( GL_DEPTH_TEST );

	/* Turn on lighting in our scene */
	glEnable( GL_LIGHTING );

	/* Have the system automatically normalize all the normals */
	glEnable( GL_NORMALIZE );

	/* Turn on the default light */
	glEnable( GL_LIGHT0 );

	glMatrixMode( GL_PROJECTION );
	aspect = (GLfloat) width / height;
	gluPerspective( fovy, aspect, near, far );
	glMatrixMode( GL_MODELVIEW );

	radius = 8.0;
	latitude = 60.0;
	longitude = 0.0;
}

GLvoid
drawScene( GLvoid )
{
	/* Define a few materials properties */
	GLfloat   redAmbient[] = { 0.3, 0.1, 0.1, 1.0 };
	GLfloat   redDiffuse[] = { 1.0, 0.0, 0.0, 1.0 };
	GLfloat   blueAmbient[] = { 0.1, 0.1, 0.3, 1.0 };
	GLfloat   blueDiffuse[] = { 0.0, 0.0, 1.0, 1.0 };
	GLfloat   yellowAmbient[] = { 0.1, 0.1, 0.0, 1.0 };
	GLfloat   yellowDiffuse[] = { 1.0, 1.0, 0.0, 1.0 };
	GLfloat   yellowEmission[] = { 0.6, 0.6, 0.0, 1.0 };
	GLfloat   whiteSpecular[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat   greenSpecular[] = { 0.0, 1.0, 0.0, 1.0 };

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glPushMatrix();

		polarView( radius, 0, latitude, longitude );

		drawAxes();

		/* Save the current lighting state ( which is default
		 *   values ) 
		 */
		glPushAttrib( GL_LIGHTING_BIT );

			/* Set the materials for a shiny red material */
			glMaterialfv( GL_FRONT, GL_AMBIENT, redAmbient );
			glMaterialfv( GL_FRONT, GL_DIFFUSE, redDiffuse );
			glMaterialfv( GL_FRONT, GL_SPECULAR, whiteSpecular );
			glMaterialf( GL_FRONT, GL_SHININESS, 100.0 );

			glPushMatrix();
				glTranslatef( -2.0, 1.5, 0.0 );
				auxSolidSphere( 1.0 );
			glPopMatrix();

		/* Restore the saved lighting state */
		glPopAttrib();

		/* Save the current lighting state */
		glPushAttrib( GL_LIGHTING_BIT );

			/* Set materials for a dull blue material with
			 *   green highlights */
			glMaterialfv( GL_FRONT, GL_AMBIENT, blueAmbient );
			glMaterialfv( GL_FRONT, GL_DIFFUSE, blueDiffuse );
			glMaterialfv( GL_FRONT, GL_SPECULAR, greenSpecular );
			glMaterialf( GL_FRONT, GL_SHININESS, 30.0 );

			glPushMatrix();
				glTranslatef( 2.5, 0.0, 0.0 );
				auxSolidTorus( 0.25, 0.75 );
			glPopMatrix();

		glPopAttrib();

		glPushAttrib( GL_LIGHTING_BIT );

			/* Set materials for a yellow glowing material with 
			 *    no specular highlights */
			glMaterialfv( GL_FRONT, GL_AMBIENT, yellowAmbient );
			glMaterialfv( GL_FRONT, GL_DIFFUSE, yellowDiffuse );
			glMaterialfv( GL_FRONT, GL_EMISSION, yellowEmission );
			glMaterialf( GL_FRONT, GL_SHININESS, 2.0 );

			glPushMatrix();
				glTranslatef( 0.0, 2.0, 2.0 );
				auxSolidCube( 0.5 );
			glPopMatrix();

		glPopAttrib();

	glPopMatrix();

	auxSwapBuffers();
}

GLvoid
startMove( AUX_EVENTREC *event )
{
	auxMouseFunc( AUX_MOUSELOC, NULL, mouseMove );
	xStart = event->data[AUX_MOUSEX];
	yStart = event->data[AUX_MOUSEY];
}

GLvoid
stopMove( AUX_EVENTREC *event )
{
	auxDeleteMouseFunc( AUX_MOUSELOC, NULL, mouseMove );
}

GLvoid
mouseMove( AUX_EVENTREC *event )
{
	GLint     xPos, yPos;
	GLint     xOffset, yOffset;

	xPos = event->data[AUX_MOUSEX];
	yPos = event->data[AUX_MOUSEY];
	xOffset = xPos - xStart;
	yOffset = yPos - yStart;
	longitude += xOffset;
	latitude -= yOffset;
	xStart = xPos;
	yStart = yPos;
}

GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat     aspect;
	GLfloat     left, right, bottom, top;

	glViewport( 0, 0, width, height );
	aspect = (GLfloat) width / height;
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( fovy, aspect, near, far );
	glMatrixMode( GL_MODELVIEW );
}

void
polarView( GLdouble radius, GLdouble twist,  GLdouble latitude,
				GLdouble longitude )
{
	glTranslated( 0.0, 0.0, -radius);
	glRotated( -twist, 0.0, 0.0, 1.0 );
	glRotated( -latitude, 1.0, 0.0, 0.0 );
	glRotated( longitude, 0.0, 0.0, 1.0 );
}

void
printHelp( char *progname )
{
	fprintf(stdout, "\n%s\n\n\\
	Escape key			- exit the program\n\
	Left Mousebutton, down		- start moving the eye\n\
	Left Mousebutton, up		- stop moving the eye\n\
	Mouse pointer location		- position eye\n\n", progname);
}

GLvoid
drawAxes( GLvoid )
{
	glPushAttrib( GL_LIGHTING_BIT|GL_LINE_BIT);
		glLineWidth( 3.0 );
		glDisable(GL_LIGHTING );
		glBegin( GL_LINES );
			/* color X axis red */
			glColor3f(  1.0, 0.0, 0.0 );
			glVertex3f( 0.0, 0.0, 0.0 );
			glVertex3f( 1.0, 0.0, 0.0 );

			/* color Y axis green */
			glColor3f(  0.0, 1.0, 0.0 );
			glVertex3f( 0.0, 0.0, 0.0 );
			glVertex3f( 0.0, 1.0, 0.0 );

			/* color Z axis blue */
			glColor3f(  0.0, 0.0, 1.0 );
			glVertex3f( 0.0, 0.0, 0.0 );
			glVertex3f( 0.0, 0.0, 1.0 );
		glEnd();
	glPopAttrib();
}
