/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*
 * spotlight.c - spotlight shining on a stack of flat grids
 *
 *		Escape key			- exit the program
 *		SPACEBAR key			- wireframe/lit/smooth grid
 */

#include <aux.h>
#include <math.h>
#include <stdio.h>

#define GRIDPOINTS 41
#define WIRE 0
#define LIT_FLAT 1
#define LIT_SMOOTH 2
#define POINTS 3

/* Function Prototypes */

GLvoid  initialize( GLvoid );
GLvoid  drawScene( GLvoid );
GLvoid  resize( GLsizei, GLsizei );
GLvoid  initGrid( GLvoid );
GLvoid  drawGridWire( GLvoid );
GLvoid  drawGridFilled( GLvoid );
GLvoid  setDrawStyle( GLvoid );
GLvoid  initLighting( GLvoid );

void  printHelp( char * );

/* Global Variables */
GLfloat			latitude, longitude, radius;

GLfloat			fovy = 60.0;	/* Field of view in Y angle */
GLfloat			near = 3.0;	/* Near clipping plane location */
GLfloat			far  = 12.0;	/* Far clipping plane location */

GLint			drawStyle = LIT_SMOOTH;
GLfloat			grid[GRIDPOINTS][GRIDPOINTS][3] = { 0 };

void
main(int argc, char *argv[])
{
	initialize();

	printHelp(argv[0]);

	auxMainLoop( drawScene );
}

GLvoid
initialize( GLvoid )
{
	GLfloat    aspect;
	GLsizei    width, height;

	auxGetScreenSize( &width, &height );
	auxInitPosition( width / 4, height / 4, width / 2,
		height / 2 );
	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH | AUX_DOUBLE );
	auxInitWindow( "spotlight     SPACEBAR--toggle filled/smooth/wireframe" );

	auxAnimation( AUX_ON );
	auxReshapeFunc( resize );
	auxKeyFunc( AUX_ESCAPE, auxQuit );
	auxKeyFunc( AUX_SPACE, setDrawStyle );

	glClearColor( 0.0, 0.0, 0.0, 1.0 );

	glEnable( GL_DEPTH_TEST );

	glMatrixMode( GL_PROJECTION );
	aspect = (GLfloat) width / height;
	gluPerspective( fovy, aspect, near, far );
	glMatrixMode( GL_MODELVIEW );

	glShadeModel( GL_FLAT );

	radius = 4.5;
	latitude = -25.0;
	longitude = 0.0;

	initLighting();
	initGrid();
}

/* Initialize material property, light source, lighting model, 
 * and depth buffer.
 */
GLvoid
initLighting( GLvoid )
{
	GLfloat mat_ambient[] = { 0.25, 0.05, 0.4, 1.0 };
	GLfloat mat_diffuse[] = { 0.5, 0.1, 0.8, 1.0 };

	/* mat_specular and mat_shininess are NOT default values */
	GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat mat_shininess[] = { 10.0 };

	GLfloat lmodel_ambient[] = { 0.2, 0.2, 0.2, 1.0 };

	glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
	glShadeModel( GL_SMOOTH );

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
}

void
printHelp( char *progname )
{
	fprintf(stdout, "\n%s\n\n\\
	Escape key		- exit the program\n\
	SPACEBAR		- filled/smooth/wireframe grid\n\n", progname);
}

void
polarView( GLdouble radius, GLdouble twist,  GLdouble latitude,
				GLdouble longitude )
{
	glTranslated( 0.0, 0.0, -radius);
	glRotated( -twist, 0.0, 0.0, 1.0 );
	glRotated( -latitude, 1.0, 0.0, 0.0 );
	glRotated( longitude, 0.0, 0.0, 1.0 );
}


GLvoid
setDrawStyle( GLvoid )
{
	if (++drawStyle > POINTS)
		drawStyle = WIRE;

	switch (drawStyle)
	{
		case WIRE:
		case POINTS:
			glEnable(GL_LIGHTING);
			glEnable(GL_LIGHT0);
			break;
		case LIT_FLAT:
			glShadeModel( GL_FLAT );
			glEnable(GL_LIGHTING);
			glEnable(GL_LIGHT0);
			break;
		case LIT_SMOOTH:
			glShadeModel( GL_SMOOTH );
			glEnable(GL_LIGHTING);
			glEnable(GL_LIGHT0);
			break;
		default:
			break;
	}
}

GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat     aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLfloat) width / height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	aspect = (GLfloat) width / height;
	gluPerspective( fovy, aspect, near, far );
	glMatrixMode( GL_MODELVIEW );
}

GLvoid
drawScene( GLvoid )
{
	int i;
	static float spin = 0.0;

	GLfloat spot_dir[] = { 0.1, -1.0, 0.0 };
	GLfloat spot_cutoff = 10.0;
	GLfloat spot_exp = 128.0;

	GLfloat light_position[] = { 0.0, 2.5, 0.0, 1.0 };

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glPushMatrix();
		polarView( radius, 0, latitude, longitude );

		glPushMatrix();
			glRotatef( spin, 0.0, 1.0, 0.0 );
			glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, spot_dir);
			glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, spot_cutoff);
			glLightf(GL_LIGHT0, GL_SPOT_EXPONENT, spot_exp);
			glLightfv(GL_LIGHT0, GL_POSITION, light_position);
		glPopMatrix();

		for( i = -2; i < 3; i++) {
			glPushMatrix();
				glTranslatef( 0.0, i * 0.5, 0.0);
				if (drawStyle == WIRE)
					drawGridWire();
				else
					drawGridFilled();
			glPopMatrix();
		}
	glPopMatrix();

	auxSwapBuffers();

	spin = fmodf( spin + 3.0, 360.0 );
}

GLvoid
initGrid( GLvoid )
{
	int i, j;
	GLfloat     x, z, spacing;

	/* Create a flat grid composed of points with y = 0 */

	/* spacing = distance between adjacent grid points in x & z */
	spacing = 2.0 / (float)(GRIDPOINTS - 1);

	/* x and z range from -1.0 to 1.0 by spacing interval */
	for ( x = -1.0, i = 0; i < GRIDPOINTS; x += spacing, i++ ) {
		for ( z = -1.0, j = 0; j < GRIDPOINTS; z += spacing, j++ ) {
			grid[i][j][0] = x;
			grid[i][j][1] = 0.0;
			grid[i][j][2] = z;
		}
	}
}

GLvoid
drawGridWire( GLvoid )
{
	register int i, j;

	/* draw vertical lines connecting grid points */
	for ( i = 0; i < (GRIDPOINTS); i++ ) {
		glBegin( GL_LINE_STRIP );
		for ( j = 0; j < (GRIDPOINTS); j++ ) {
			glVertex3fv( &grid[i][j][0] );
		}
		glEnd();
	}

	/* draw horizontal lines connecting grid points */
	for ( j = 0; j < (GRIDPOINTS); j++ ) {
		glBegin( GL_LINE_STRIP );
		for ( i = 0; i < (GRIDPOINTS); i++ ) {
			glVertex3fv( &grid[i][j][0] );
		}
		glEnd();
	}
}

GLvoid
drawGridFilled( GLvoid )
{
	register int i, j;

	glNormal3f( 0.0, 1.0, 0.0 );
	
	/* Draw quad strips connecting grid points. 
	 * Grid is drawn column by column, 
	 * where each column is one quad strip.
	 * Every pair of vertices after first two vertices
	 * adds a new quad to the strip.
	 */
	for ( i = 0; i < (GRIDPOINTS - 1); i++ ) {
		if (drawStyle == POINTS)
			glBegin( GL_POINTS );
		else
			glBegin( GL_QUAD_STRIP );
		for ( j = 0; j < (GRIDPOINTS); j++ ) {
			glVertex3fv( &grid[i][j][0] );
			glVertex3fv( &grid[i + 1][j][0] );				
		}
		glEnd();
	}
}
